@@ -4,6 +4,8 @@ import time
 from multiprocessing import Process, Queue
 from threading import Timer
 from typing import Any, Callable, Dict, Generic, Iterable, List, Optional, TypeVar
+
+from olympipe.pipes.reduce import ReducePipe
 from .pipes.generic import GenericPipe
 from .pipes.task import TaskPipe
 from .pipes.filter import FilterPipe
@@ -143,11 +145,19 @@ class Pipeline(Generic[R]):
         self.dispatcher.dispatch_to([q])
         return q
 
-    # def wait_and_reduce(self,
-    #                     accumulator: T,
-    #                     reducer: Callable[[R, T], T]
-    #                     ) -> T:
-    #     self.
+    def reduce(self, accumulator: T, reducer: Callable[[R, T], T]) -> "Pipeline[T]":
+        target_queue: "Queue[R]" = Pipeline.get_new_queue()
+        output_pipe: "Pipeline[T]" = Pipeline()
+
+        p = ReducePipe(target_queue, output_pipe._source, accumulator, reducer)
+        self._register(p)
+        self.dispatcher.dispatch_to([target_queue])
+        return output_pipe
+
+    def wait_and_reduce(self, accumulator: T, reducer: Callable[[R, T], T]) -> "T":
+        o = self.reduce(accumulator, reducer)
+        [[res]] = Pipeline.wait_for_all_results([o])
+        return res
 
     @staticmethod
     def wait_for_all_completions(
@@ -194,4 +204,5 @@ class Pipeline(Generic[R]):
         return Pipeline.wait_for_all_results([self, *other_pipes])
 
     def wait_for_result(self) -> List[R]:
-        return Pipeline.wait_for_all_results([self])[0]
+        [res] = Pipeline.wait_for_all_results([self])
+        return res
